//
// programming note for router SOC
// DO NOT USE fscanf/fprintf
//

#define MAX_IPC_MSG_BUF 20

typedef struct
{
    long mtype;
    char mtext[MAX_IPC_MSG_BUF];
} msgbuf;

enum {
    IPC_CLIENT_MSG_Q_ID = 1,
    IPC_START_AUTO_DET,
    IPC_LINK_STATE,
    IPC_ADD_PVC,
//    IPC_DEL_PVC,
    IPC_DEL_ALL_PVC,
    IPC_STOP_AUTO_DET,
    IPC_ATE_ADSL_SHOW,
    IPC_ATE_SET_ADSL_MODE,
    IPC_ATE_ADSL_DISP_PVC,
    IPC_ATE_ADSL_QUIT_DRV
};


//
// if autodetect enabled, adsl status polling MUST be disabled (no timer function)
// This is because multiple send and wait resp is not allowed
//

#ifndef WIN32
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/time.h>
#include <signal.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#else
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#define msgsnd(...) 0
#define msgrcv(...) 0
#define EINTR 0
#endif



static int m_Cnt = 0;

extern void UpdateAllAdslSts(int cnt, int from_ate);
extern void DispAllPvc();
extern int AddPvc(int idx, int vlan_id, int vpi, int vci, int encap, int mode);
extern int DelAllPvc();
extern void GetLinkSts(char* buf, int max_len);
extern void myprintf(const char *fmt, ...);
extern void SleepMs(int ms);
extern int SetAdslMode(int EnumAdslModeValue, int FromAteCmd);
extern int SetAdslType(int EnumAdslTypeValue, int FromAteCmd);

#ifndef WIN32
volatile sig_atomic_t m_start_polling=0;
volatile sig_atomic_t m_stop_polling_ok=0;
volatile sig_atomic_t m_req_auto_det_pvc=0;
#else
volatile int m_start_polling=0;
volatile int m_stop_polling_ok=0;
volatile int m_req_auto_det_pvc=0;
#endif

void enable_polling(void)
{
    m_start_polling = 1;
}

void disable_polling(void)
{
    m_req_auto_det_pvc = 1;            
}

void wait_polling_stop(void)
{
    while (1)
    {
        if (m_stop_polling_ok == 1)
        {
            break;
        } 
        SleepMs(10);
    }
    m_stop_polling_ok = 0;
}


void polling_tc_info(int signo) 
{ 
    if (m_req_auto_det_pvc)
    {
        m_start_polling = 0;
        m_stop_polling_ok = 1;
    }
    else
    {
        if (m_start_polling)
        {
            UpdateAllAdslSts(m_Cnt,0);
        }
    }
    m_Cnt++;
} 

#ifndef WIN32

void init_sigaction(void) 
{ 
    struct sigaction act;
    act.sa_handler=polling_tc_info; 
    act.sa_flags=0; 
    sigemptyset(&act.sa_mask); 
    sigaction(SIGALRM,&act,NULL); 
    myprintf("process id is %d\n", getpid());
}


void init_time() 
{ 
    struct itimerval value; 
    value.it_value.tv_sec=5; 
    value.it_value.tv_usec=0; 
    value.it_interval=value.it_value; 
    setitimer(ITIMER_REAL,&value,NULL); 
}


#endif


void SetPollingTimer()
{
#ifndef WIN32
    init_sigaction(); 
    init_time(); 
#endif    
}

static int m_msqid_to_d=0;
static int m_msqid_from_d=0;

int CreateMsgQ()
{
#ifndef WIN32
    char buf[32];
    FILE* fp;    
    if ((m_msqid_to_d=msgget(IPC_PRIVATE,0700))<0)
    {
        myprintf("msgget err\n");
        return -1;
    }
    else
    {
        myprintf("msgget ok\n");
    }        
    fp = fopen("/tmp/tc_resp_to_d.txt","wb");
    if (fp == NULL) return -1;
    sprintf(buf,"%d",m_msqid_to_d);
    fputs(buf,fp);
    fclose(fp);
    /*
    if ((m_msqid_from_d=msgget(IPC_PRIVATE,0700))<0)
    {
        myprintf("msgget err\n");
        return -1;
    }
    else
    {
        myprintf("msgget ok\n");
    }    
    fp = fopen("/tmp/tc_resp_from_d.txt","wb");
    if (fp == NULL) return -1;
    sprintf(buf,"%d",m_msqid_from_d);
    fputs(buf,fp);
    fclose(fp);
    */
#endif    
    return 0;
}

int RcvMsgQ()
{
    int infolen;
    int bAskQuit = 0;
    msgbuf send_buf;        
    msgbuf receive_buf;    
    if((infolen=msgrcv(m_msqid_to_d,&receive_buf,MAX_IPC_MSG_BUF,0,0))<0)
    {
        if (errno == EINTR)
        {
          return 0;
        }
        else
        {
          myprintf("TP_INIT:msgrcv err\n");
          return -1;
        }
    }
    else
    {
        //printf("msgrcv ok\n");
        int bSendResp = 1;
        if (IPC_CLIENT_MSG_Q_ID == receive_buf.mtype)
        {
            int* pInt;
            pInt=(int*)(&receive_buf.mtext[0]);                    
            myprintf("TP_INIT:IPC_CLIENT_MSG_Q_ID\n");        
            m_msqid_from_d = *pInt;
            myprintf("%d\n",m_msqid_from_d);                    
            bSendResp = 0;
        }
        else if (IPC_START_AUTO_DET == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_START_AUTO_DET\n");
            disable_polling();
            wait_polling_stop();            
            send_buf.mtype=IPC_START_AUTO_DET;
            strcpy(send_buf.mtext,"Start Done");        
        }
        else if (IPC_ADD_PVC == receive_buf.mtype)
        {
            unsigned short idx;
            unsigned short vlan_id;
            unsigned short vpi;
            unsigned short vci;
            unsigned short encap;
            unsigned short mode;
            unsigned short* pWord;
            pWord=(unsigned short*)(&receive_buf.mtext[0]);            
            int add_ret;            
            myprintf("TP_INIT:IPC_ADD_PVC\n");                        
            idx = *pWord++;
            vlan_id = *pWord++;
            vpi = *pWord++;
            vci = *pWord++;
            encap = *pWord++;
            mode = *pWord;
            myprintf("ADDPVC:%d %d %d %d %d %d\n",idx,vlan_id,vpi,vci,encap,mode);
            add_ret = AddPvc(idx,vlan_id,vpi,vci,encap,mode);
            send_buf.mtype=IPC_ADD_PVC;
            if (add_ret == 0) strcpy(send_buf.mtext,"OK");
            else strcpy(send_buf.mtext,"FAIL");
        }        
        else if (IPC_DEL_ALL_PVC == receive_buf.mtype)
        {
            int del_ret;
            myprintf("TP_INIT:IPC_DEL_ALL_PVC\n");                        
            del_ret = DelAllPvc();
            send_buf.mtype=IPC_DEL_ALL_PVC;
            if (del_ret == 0) strcpy(send_buf.mtext,"OK");
            else strcpy(send_buf.mtext,"FAIL");
        }
        else if (IPC_LINK_STATE == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_LINK_STATE\n");                                
            send_buf.mtype=IPC_LINK_STATE;        
            GetLinkSts(send_buf.mtext, MAX_IPC_MSG_BUF);
/*            
"down"
"wait for initializi"
"initializing"
"up"
*/
        }
        else if (IPC_STOP_AUTO_DET == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_STOP_AUTO_DET\n");                                        
            send_buf.mtype=IPC_STOP_AUTO_DET;
            strcpy(send_buf.mtext,"Stop Done");
            enable_polling();
        }                
        else if (IPC_ATE_ADSL_SHOW == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_ATE_ADSL_SHOW\n");                                        
            UpdateAllAdslSts(0,1);
            send_buf.mtype=IPC_ATE_ADSL_SHOW;
            strcpy(send_buf.mtext,"AteSetDone");           
        }                
        else if (IPC_ATE_ADSL_QUIT_DRV == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_ATE_ADSL_QUIT_DRV\n");                                        
            bAskQuit = 1;
            send_buf.mtype=IPC_ATE_ADSL_QUIT_DRV;
            strcpy(send_buf.mtext,"AteSetDone");           
        }           
        else if (IPC_ATE_ADSL_DISP_PVC == receive_buf.mtype)
        {
            myprintf("TP_INIT:IPC_ATE_ADSL_DISP_PVC\n");                                        
            DispAllPvc();
            send_buf.mtype=IPC_ATE_ADSL_DISP_PVC;
            strcpy(send_buf.mtext,"AteSetDone");           
        }                        
        else if (IPC_ATE_SET_ADSL_MODE == receive_buf.mtype)
        {
            unsigned short mode;
            unsigned short type;
            unsigned short* pWord;
            pWord=(unsigned short*)(&receive_buf.mtext[0]);            
            int set_ret_mode, set_ret_type;            
            myprintf("TP_INIT:IPC_ATE_SET_ADSL_MODE\n");                                        
            mode = *pWord++;
            type = *pWord++;
            myprintf("SET MODE:%d %d\n",mode,type);
            set_ret_mode = SetAdslMode(mode,1);
            set_ret_type = SetAdslType(type,1);
            send_buf.mtype=IPC_ATE_SET_ADSL_MODE;
            if (set_ret_mode == 0 && set_ret_type) strcpy(send_buf.mtext,"OK");
            else strcpy(send_buf.mtext,"FAIL");            
        }                                
        
        if (bSendResp)
        {
            if(msgsnd(m_msqid_from_d,&send_buf,MAX_IPC_MSG_BUF,0)<0)
            {
                myprintf("msgsnd fail\n");
                return -1;
            }
        }
    }
  
    if (bAskQuit == 1) return 1;
    return 0;
}


// we need care about ralink ver / fw ver interoperation problem

// accel auto det,
// add multiple pvc once , and detect the PPPOE multiple time

//ipalias
//ifconfig eth2.2:0 194.255.255.1 netmask 255.255.255.0 
// ifconfig eth2.2:0 up
//ping 194.255.255.254 -t
// ftpget  192.168.1.15 /tmp/ras ras  
//tftp -p -l ras 194.255.255.254
// ftpput  192.168.1.15 chk /tmp/ras

